<?php

namespace IMATHUZH\Qfq\Core\Form\FormElement;

use IMATHUZH\Qfq\Core\Form\Form;
use IMATHUZH\Qfq\Core\Helper\HelperFormElement;
use IMATHUZH\Qfq\Core\Helper\Support;

class CheckboxFormElement extends AbstractFormElement {
    public array $itemKeys = array();
    public array $itemValues = array();

    /**
     * @param $attributes
     * @param Form|null $form
     * @throws \CodeException
     * @throws \DbException
     * @throws \DownloadException
     * @throws \PhpOffice\PhpSpreadsheet\Exception
     * @throws \PhpOffice\PhpSpreadsheet\Reader\Exception
     * @throws \PhpOffice\PhpSpreadsheet\Writer\Exception
     * @throws \Twig\Error\LoaderError
     * @throws \Twig\Error\RuntimeError
     * @throws \Twig\Error\SyntaxError
     * @throws \UserFormException
     * @throws \UserReportException
     */
    public function __construct($attributes, ?Form $form = null) {
        parent::__construct($attributes, $form);

        $this->setKeysAndValues();
        $this->attributes[FE_CHECKBOX_MODE] = $this->getCheckBoxMode();

        if ($this->attributes[FE_CHECKBOX_MODE] == FE_CHECKBOX_MODE_MULTI_STATE) {
            $this->setMultiStateCheckBoxValues();
        } else {
            $this->setDefaults();
        }

    }

    /**
     * Fills $this->itemKeys and $this->itemValues
     *
     * @return void
     * @throws \CodeException
     * @throws \UserFormException
     * @throws \UserReportException
     */
    private function setKeysAndValues(): void {
        HelperFormElement::getKeyValueListFromSqlEnumSpec($this->attributes, $this->itemKeys, $this->itemValues);
    }

    /**
     * Set meaningful default values
     *
     * @return void
     * @throws \CodeException
     * @throws \DbException
     * @throws \UserFormException
     * @throws \UserReportException
     */
    private function setDefaults(): void {
        $this->htmlAttributes['type'] = 'checkbox';
        $this->attributes[FE_CHECKBOX_MODE] = $this->getCheckBoxMode();

        if ($this->attributes[FE_CHECKBOX_MODE] == FE_CHECKBOX_MODE_MULTI_STATE) {

            $this->setMultiStateCheckBoxValues();

        } elseif ($this->attributes[FE_CHECKBOX_MODE] != 'multi') {
            // Checked
            if (!isset($this->attributes[FE_CHECKBOX_CHECKED])) {
                if (isset($this->itemKeys[0])) {
                    // First element in $itemKey list
                    $this->attributes[FE_CHECKBOX_CHECKED] = $this->itemKeys[0];
                } else {
                    // Take column default value
                    $this->attributes[FE_CHECKBOX_CHECKED] = $this->store->getVar($this->attributes[FE_NAME], STORE_TABLE_DEFAULT);
                }
            }

            // Unchecked
            if (!isset($this->attributes[FE_CHECKBOX_UNCHECKED])) {
                if (isset($this->itemKeys[1])) {
                    $this->attributes[FE_CHECKBOX_UNCHECKED] = ($this->itemKeys[0] === $this->attributes[FE_CHECKBOX_CHECKED]) ? $this->itemKeys[1] : $this->itemKeys[0];
                } else {
                    $this->attributes[FE_CHECKBOX_UNCHECKED] = '';
                }
            }

            if ($this->attributes[FE_CHECKBOX_CHECKED] === $this->attributes[FE_CHECKBOX_UNCHECKED]) {
                throw new \UserFormException('FormElement: type=checkbox - checked and unchecked can\'t be the same: ' . $this->attributes[FE_CHECKBOX_CHECKED], ERROR_CHECKBOX_EQUAL);
            }
        }

        // Required Error
        if (isset($this->attributes[F_FE_DATA_REQUIRED_ERROR])) {
            $this->htmlAttributes[F_FE_DATA_REQUIRED_ERROR] = $this->attributes[F_FE_DATA_REQUIRED_ERROR];
        }

    }

    /**
     * Sets the options for a MultiState Checkbox by ether DB ENUM/SET or param multiCheckBoxOptions
     *
     * example:
     * multiCheckBoxOptions = VALUE:LABEL:FONT AWESOME ICON:ICON COLOR:BACKGROUND COLOR,no:No:fa-times:white:#2196f3,
     * @throws \UserReportException
     * @throws \UserFormException
     * @throws \CodeException
     */
    private function setMultiStateCheckBoxValues() {
        $checkboxStates = empty($this->attributes[FE_MULTI_CHECKBOX_ITEMS]) ? False : $this->attributes[FE_MULTI_CHECKBOX_ITEMS];
        $options = [];
        if (!$checkboxStates) {
            // get values from DB Column
            $this->setKeysAndValues();
            // Fill options
            for ($i = 0; $i < count($this->itemKeys); $i++) {
                // First Option should have white background and no Icon
                if ($i == 0) {
                    $options[] = [
                        "value" => $this->itemValues[$i],
                        "label" => $this->itemKeys[$i],
                        "icon" => "",
                        "iconColor" => "",
                        "color" => "white"
                    ];
                } else {
                    $options[] = [
                        "value" => $this->itemValues[$i],
                        "label" => $this->itemKeys[$i],
                        "icon" => "fa-check",
                        "iconColor" => "white",
                        "color" => "#2196f3"
                    ];
                }
            }
        } else {

            $rawOptions = explode(',', $checkboxStates);
            // Fill Options
            foreach ($rawOptions as $option) {
                list($value, $label, $icon, $iconColor, $color) = explode(':', $option);
                $options[] = [
                    "value" => $value,
                    "label" => $label,
                    "icon" => $icon,
                    "iconColor" => $iconColor,
                    "color" => $color
                ];
            }

        }
        $this->attributes[FE_MULTI_CHECKBOX_ITEMS] = $options;
    }

    /** Get the state of the checkbox.
     * @return string
     */
    private function getCheckBoxMode(): string {
        // Checkbox mode is set
        if (isset($this->attributes[FE_CHECKBOX_MODE])) {
            return $this->attributes[FE_CHECKBOX_MODE];
        }
        // checkBoxState param is set expected 'multiState' checkbox
        if (isset($this->attributes[FE_MULTI_CHECKBOX_ITEMS])) {
            return FE_CHECKBOX_MODE_MULTI_STATE;
        }
        // more than 2 items expected 'multi' checkbox
        if (count($this->itemKeys) > 2) {
            return FE_CHECKBOX_MODE_MULTI;
        }
        // default single checkbox
        return FE_CHECKBOX_MODE_SINGLE;
    }
}